use std::{collections::HashSet, fs};

const ROW: i32 = 2000000;

pub fn solve() {
    let path = "res/15/input.txt";

    let contents = fs::read_to_string(path).expect("File read error");

    let sensors: Vec<((i32, i32), (i32, i32), i32)> = contents
        .lines()
        .filter(|line| *line != "")
        .map(|line| {
            let line = line
                .replace("x", "")
                .replace("y", "")
                .replace("=", "")
                .replace(":", "")
                .replace(",", "");

            let words: Vec<&str> = line.split_whitespace().collect();

            let x1 = words[2].parse::<i32>().unwrap();
            let y1 = words[3].parse::<i32>().unwrap();
            let x2 = words[8].parse::<i32>().unwrap();
            let y2 = words[9].parse::<i32>().unwrap();
            let rad = x1.abs_diff(x2) as i32 + y1.abs_diff(y2) as i32;
            ((x1, y1), (x2, y2), rad)
        })
        .collect();

    let mut row_map = HashSet::<i32>::new();

    sensors.iter().for_each(|sensor| {
        let ((x1, y1), (x2, y2), rad) = sensor;

        let y_diff = y1.abs_diff(ROW) as i32;
        let x_diff = rad - y_diff;
        if x_diff < 0 {
            return;
        }

        (x1 - x_diff..=x1 + x_diff).for_each(|i| {
            row_map.insert(i);
        })
    });

    sensors.iter().for_each(|sensor| {
        let ((_, _), (x2, y2), _) = sensor;

        if *y2 == ROW && row_map.contains(&x2) {
            row_map.remove(&x2);
        }
    });

    let res = row_map.len();
    println!("Result 1: {res}");

    let mut sen_tup = Vec::<((i32, i32, i32), (i32, i32, i32))>::new();
    let mut up_outlines = Vec::<i32>::new();
    let mut down_outlines = Vec::<i32>::new();
    sensors.iter().for_each(|s1| {
        sensors.iter().for_each(|s2| {
            if s1 != s2 {
                let ((x1, y1), (_, _), rad1) = *s1;
                let ((x2, y2), (_, _), rad2) = *s2;
                let dist = x1.abs_diff(x2) as i32 + y1.abs_diff(y2) as i32;
                if dist == rad1 + rad2 + 1 {
                    sen_tup.push(((x1, y1, rad1), (x2, y2, rad2)));
                    // s2 in quadrant 1
                    if x1 - x2 < 0 && y1 - y2 > 0 {
                        down_outlines.push(y1 - (x1 + rad1) - 1)
                    }
                    // s2 in quadrant 3
                    else if x1 - x2 > 0 && y1 - y2 < 0 {
                        down_outlines.push(y2 - (x2 + rad2) - 1)
                    }
                    // s2 in quadrant 2
                    else if x1 - x2 > 0 && y1 - y2 > 0 {
                        up_outlines.push(y2 + (x2 + rad2) + 1)
                    }
                    // s2 in quadrant 4
                    else if x1 - x2 < 0 && y1 - y2 < 0 {
                        up_outlines.push(y1 + (x1 + rad1) + 1)
                    }
                }
            }
        })
    });

    let mut intersections = Vec::<(i32, i32)>::new();
    up_outlines.iter().for_each(|up_line| {
        down_outlines.iter().for_each(|down_line| {
            if up_line > down_line && up_line % 2 == down_line % 2 {
                let intersection = down_line + (up_line - down_line) / 2;
                // TODO continue
            }
        })
    });

    sen_tup.iter().for_each(|tup| {});
}
